package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.ActCustomForm;
import cn.com.ctop.common.module.service.IActCustomFormService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

 /**
 * 自定义表单信息
 * @author jeecg-boot
 * @date   2020-08-25
 * @version V1.0
 */
@Slf4j
@Api(tags="自定义表单信息")
@RestController
@RequestMapping("/ctop/actCustomForm")
public class ActCustomFormController {
	@Autowired
	private IActCustomFormService actCustomFormService;

	/**
	  * 分页列表查询
	 * @param actCustomForm
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@ApiOperation(value="自定义表单信息-分页列表查询", notes="自定义表单信息-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ActCustomForm>> queryPageList(ActCustomForm actCustomForm,
													  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
													  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
													  HttpServletRequest req) {
		Result<IPage<ActCustomForm>> result = new Result<>();
		QueryWrapper<ActCustomForm> queryWrapper = QueryGenerator.initQueryWrapper(actCustomForm, req.getParameterMap());
		Page<ActCustomForm> page = new Page<ActCustomForm>(pageNo, pageSize);
		IPage<ActCustomForm> pageList = actCustomFormService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param actCustomForm
	 * @return
	 */
	@ApiOperation(value="自定义表单信息-添加", notes="自定义表单信息-添加")
	@PostMapping(value = "/add")
	public Result<ActCustomForm> add(@RequestBody ActCustomForm actCustomForm) {
		Result<ActCustomForm> result = new Result<>();
		try {
			LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
			actCustomForm.setUserId(user.getId());
			actCustomForm.setStatus(1);
			actCustomForm.setCreateTime(new Date());
			actCustomForm.setUpdateTime(new Date());
			actCustomFormService.save(actCustomForm);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param actCustomForm
	 * @return
	 */
	@ApiOperation(value="自定义表单信息-编辑", notes="自定义表单信息-编辑")
	@PutMapping(value = "/edit")
	public Result<ActCustomForm> edit(@RequestBody ActCustomForm actCustomForm) {
		Result<ActCustomForm> result = new Result<ActCustomForm>();
		ActCustomForm actCustomFormEntity = actCustomFormService.getById(actCustomForm.getId());
		if(actCustomFormEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = actCustomFormService.updateById(actCustomForm);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@ApiOperation(value="自定义表单信息-通过id删除", notes="自定义表单信息-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			actCustomFormService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@ApiOperation(value="自定义表单信息-批量删除", notes="自定义表单信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<ActCustomForm> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<ActCustomForm> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.actCustomFormService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@ApiOperation(value="自定义表单信息-通过id查询", notes="自定义表单信息-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ActCustomForm> queryById(@RequestParam(name="id",required=true) String id) {
		Result<ActCustomForm> result = new Result<>();
		ActCustomForm actCustomForm = actCustomFormService.getById(id);
		if(actCustomForm==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(actCustomForm);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<ActCustomForm> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              ActCustomForm actCustomForm = JSON.parseObject(deString, ActCustomForm.class);
              queryWrapper = QueryGenerator.initQueryWrapper(actCustomForm, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<ActCustomForm> pageList = actCustomFormService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "自定义表单信息列表");
      mv.addObject(NormalExcelConstants.CLASS, ActCustomForm.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("自定义表单信息列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<ActCustomForm> listActCustomForms = ExcelImportUtil.importExcel(file.getInputStream(), ActCustomForm.class, params);
              actCustomFormService.saveBatch(listActCustomForms);
              return Result.ok("文件导入成功！数据行数:" + listActCustomForms.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
