package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.TagInfo;
import cn.com.ctop.common.module.model.TagInfoTreeModel;
import cn.com.ctop.common.module.service.ITagInfoService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

 /**
 * 标签信息
 * @author jeecg-boot
 * @date   2020-08-26
 * @version V1.0
 */
@Slf4j
@RestController
@RequestMapping("/ctop/tagInfo")
public class TagInfoController {
	@Autowired
	private ITagInfoService tagInfoService;

	/**
	  * 分页列表查询
	 * @param tagInfo
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<TagInfo>> queryPageList(TagInfo tagInfo, @RequestParam(name="pageNo", defaultValue="1") Integer pageNo, @RequestParam(name="pageSize", defaultValue="100") Integer pageSize,
												HttpServletRequest req) {
		Result<IPage<TagInfo>> result = new Result<>();
		LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
		QueryWrapper<TagInfo> queryWrapper = QueryGenerator.initQueryWrapper(tagInfo, req.getParameterMap());
		Page<TagInfo> page = new Page<>(pageNo, pageSize);
		queryWrapper.eq("del_flag",0);
		IPage<TagInfo> pageList = tagInfoService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param tagInfo
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<TagInfo> add(@RequestBody TagInfo tagInfo) {
		Result<TagInfo> result = new Result<>();
		try {
			TagInfo getTag = tagInfoService.getByTagCode(tagInfo.getTagCode());
			if(null!=getTag){
				result.error500("标签编码已存在");
				return result;
			}
			LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
			tagInfo.setDelFlag(0);
			if(null == tagInfo.getStatus()){
				tagInfo.setStatus(1);
			}
			tagInfo.setUserId(user.getId());
			int level = 1;
			Long parentId = tagInfo.getParentId();
			if(null!=parentId){
				TagInfo parentTag = tagInfoService.getById(parentId);
				if(null == parentTag){
					result.error500("父级标签不存在");
					return result;
				}
				String tagCode = parentTag.getTagCode()+tagInfo.getTagCode();
				TagInfo getTag2 = tagInfoService.getByTagCode(tagCode);
				if(null!=getTag2){
					result.error500("标签编码已存在");
					return result;
				}
				tagInfo.setTagCategoryId(parentTag.getTagCategoryId());
				if(parentTag.getLevel() == 1){
					tagInfo.setFirstTagId(parentTag.getId());
				}
				if(parentTag.getLevel() == 2){
					tagInfo.setSecendTagId(parentTag.getId());
					tagInfo.setFirstTagId(parentTag.getParentId());
				}
				level = parentTag.getLevel()+1;
				tagInfo.setTagCode(tagCode);
			}
			tagInfo.setLevel(level);
			tagInfoService.save(tagInfo);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param tagInfo
	 * @return
	 */
	@PostMapping("/edit")
	public Result<TagInfo> edit(@RequestBody TagInfo tagInfo) {
		Result<TagInfo> result = new Result<>();
		TagInfo tagInfoEntity = tagInfoService.getById(tagInfo.getId());
		if(tagInfoEntity==null) {
			result.error500("未找到对应实体");
		}else {
			tagInfo.setUpdateTime(new Date());
			boolean ok = tagInfoService.updateById(tagInfo);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<TagInfo> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<TagInfo> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.tagInfoService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<TagInfo> queryById(@RequestParam(name="id",required=true) String id) {
		Result<TagInfo> result = new Result<>();
		TagInfo tagInfo = tagInfoService.getById(id);
		if(tagInfo==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(tagInfo);
			result.setSuccess(true);
		}
		return result;
	}

	@RequestMapping(value = "/queryTreeList", method = RequestMethod.GET)
    public Result<List<TagInfoTreeModel>> queryTreeList(@RequestParam(name = "categoryId",defaultValue = "0")Long categoryId) {
	    Result<List<TagInfoTreeModel>> result = new Result<>();
	    try {
			LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
            List<TagInfoTreeModel> list = tagInfoService.queryTreeList(user.getId(),categoryId);
	        result.setResult(list);
	        result.setSuccess(true);
	    } catch (Exception e) {
	        log.error(e.getMessage(),e);
	    }
	    return result;
	}
}
